package com.gsww.plugin.utils;


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * @author Wurd 2005-11-27  
 * 
 * 日期帮助类
 */
public class TimeHelper {
	
	
	private static String CurrentTime;
	private static String CurrentDate;

	public static String getCurrentDateTimeForFormat(String format) {
		java.util.Date NowDate = new java.util.Date();

		SimpleDateFormat formatter = new SimpleDateFormat(format);
		return formatter.format(NowDate);
	}

	/**
	 * 得到当前的年份
	 * 返回格式:yyyy
	 * @return String
	 */
	public static String getCurrentYear() {
		java.util.Date NowDate = new java.util.Date();

		SimpleDateFormat formatter = new SimpleDateFormat("yyyy");
		return formatter.format(NowDate);
	}

	public static String formatDate(Date date, String format) {
		return new SimpleDateFormat(format).format(date);
	}

	/**
	 * 得到当前的月份
	 * 返回格式:MM
	 * @return String
	 */
	public static String getCurrentMonth() {
		java.util.Date NowDate = new java.util.Date();

		SimpleDateFormat formatter = new SimpleDateFormat("MM");
		return formatter.format(NowDate);
	}

	/**
	 * 得到当前的日期
	 * 返回格式:dd
	 * @return String
	 */
	public static String getCurrentDay() {
		java.util.Date NowDate = new java.util.Date();

		SimpleDateFormat formatter = new SimpleDateFormat("dd");
		return formatter.format(NowDate);
	}

	/**
	 * 得到当前的时间，精确到毫秒,共19位
	 * 返回格式:yyyy-MM-dd HH:mm:ss
	 * @return String
	 */
	public static String getCurrentTime() {
		Date NowDate = new Date();
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		CurrentTime = formatter.format(NowDate);
		return CurrentTime;
	}

	/**
	 * 得到当前的日期,共10位
	 * 返回格式：yyyy-MM-dd
	 * @return String
	 */
	public static String getCurrentDate() {
		Date NowDate = new Date();
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
		CurrentDate = formatter.format(NowDate);
		return CurrentDate;
	}

	/**
	 * 得到当前日期加上某一个整数的日期，整数代表天数
	 * 输入参数：currentdate : String 格式 yyyy-MM-dd
	 * 			add_day		:  int
	 * 返回格式：yyyy-MM-dd
	 */
	public static String addDay(String currentdate, int add_day) {
		GregorianCalendar gc = null;
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
		int year, month, day;

		try {
			year = Integer.parseInt(currentdate.substring(0, 4));
			month = Integer.parseInt(currentdate.substring(5, 7)) - 1;
			day = Integer.parseInt(currentdate.substring(8, 10));

			gc = new GregorianCalendar(year, month, day);
			gc.add(GregorianCalendar.DATE, add_day);

			return formatter.format(gc.getTime());
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	/**
	 * 得到当前日期加上某一个整数的日期，整数代表天数
	 * 输入参数：currentdate : String 格式 yyyy-MM-dd
	 * 			add_day		:  int
	 * 返回格式：yyyy-MM-dd
	 */
	public static String addDay(String currentdate, int add_day,String format) {
		GregorianCalendar gc = null;
		SimpleDateFormat formatter = new SimpleDateFormat(format);
		int year, month, day;

		try {
			year = Integer.parseInt(currentdate.substring(0, 4));
			month = Integer.parseInt(currentdate.substring(5, 7)) - 1;
			day = Integer.parseInt(currentdate.substring(8, 10));

			gc = new GregorianCalendar(year, month, day);
			gc.add(GregorianCalendar.DATE, add_day);

			return formatter.format(gc.getTime());
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * @description 将格式为 yyyy-MM-dd 的日期字符串转换为一个 java.util.Date 对象
	 * @param sDate 格式为 yyyy-MM-dd 的日期字符串
	 * @return java.util.Date 对象
	 * @return Date
	 * @throws 转换失败
	 */
	public static Date parseDate(String sDate) {
		SimpleDateFormat bartDateFormat = new SimpleDateFormat("yyyy-MM-dd");

		try {
			Date date = bartDateFormat.parse(sDate);
			return date;
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
		return null;
	}

	/**
	 * @description 将格式为 yyyy-MM-dd hh:mm:ss 的日期字符串转换为一个 java.util.Date 对象
	 * @param sDate 格式为 yyyy-MM-dd hh:mm:ss 的日期字符串
	 * @return java.util.Date 对象
	 * @return Date
	 * @throws 转换失败
	 */
	public static Date parseDateTime(String sDate) {
		SimpleDateFormat bartDateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");

		try {
			Date date = bartDateFormat.parse(sDate);
			return date;
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
		return null;
	}

	/**
	 * @description 将分别为整形的年月日转换为一个Date对象
	 * @param year 年
	 * @param month 月
	 * @param day 日
	 * @return Date对象
	 * @return Date
	 * @throws 转换失败
	 */
	public static Date parseDate(Integer year, Integer month, Integer day) {
		return parseDate(new StringBuffer(10).append(year).append("-").append(month).append("-").append(day).toString());
	}

	public static String getWeekDate(String currentdate, int weekDay) {
		Calendar calendar = Calendar.getInstance();
		int wday;
		try {
			wday = calendar.get(Calendar.DAY_OF_WEEK) - 1;
			if (wday == 0) {
				wday = 7;
			}
			return addDay(currentdate, weekDay - wday);
		} catch (Exception e) {
			e.printStackTrace();
			return "";
		}
	}

	public static Date convertToDate(String date) {
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
		try {
			return formatter.parse(date);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * 给定字符串和格式，返回加一个月的串
	 * 输入："1998-11-10"，"yyyy-MM-dd"
	 * 输出："1998-12-10"
	 * @param d
	 * @param format
	 * @return
	 * @author yuzx
	 */
	public static String getStrByaddMonth(String d, String format) {
		SimpleDateFormat sdf = new SimpleDateFormat(format);
		try {
			Date date = sdf.parse(d);
			GregorianCalendar gc = new GregorianCalendar();
			gc.setTime(date);
			gc.add(Calendar.MONTH, 1);
			return sdf.format(gc.getTime());
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 给定字符串和格式，返回加一个星期的串
	 * 输入："1998-11-10"，"yyyy-MM-dd"
	 * 输出："1998-12-10"
	 * @param d
	 * @param format
	 * @return
	 * @author yuzx
	 */
	public static String getStrByaddWeek(String d, String format) {
		SimpleDateFormat sdf = new SimpleDateFormat(format);
		try {
			Date date = sdf.parse(d);
			GregorianCalendar gc = new GregorianCalendar();
			gc.setTime(date);
			gc.add(Calendar.WEEK_OF_YEAR, 1);
			return sdf.format(gc.getTime());
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	//"yy-MM-dd"
	public static int compareToDate(String src, String obj, String format) throws Exception {
		SimpleDateFormat sdf = new SimpleDateFormat(format);
		GregorianCalendar srcStartGc = new GregorianCalendar();
		GregorianCalendar srcEndGc = new GregorianCalendar();
		srcStartGc.setTime(sdf.parse(src));
		srcEndGc.setTime(sdf.parse(obj));
		System.out.println(srcStartGc.compareTo(srcEndGc));
		return srcStartGc.compareTo(srcEndGc);
	}

	/**
	 * 计算2个日期相差的月数
	 * @param s
	 * @param e
	 * @return
	 * zby
	 */
	public static int getMonth(String data1, String data2) {
		int result = 0;
		try {
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
			Date startdate = sdf.parse(data1);
			Date enddate = sdf.parse(data2);

			Calendar start = Calendar.getInstance();
			start.setTime(startdate);
			Calendar end = Calendar.getInstance();
			end.setTime(enddate);
			Calendar temp = Calendar.getInstance();
			temp.setTime(enddate);
			temp.add(Calendar.DATE, 1);

			int y = end.get(Calendar.YEAR) - start.get(Calendar.YEAR);
			int m = end.get(Calendar.MONTH) - start.get(Calendar.MONTH);

			result = y * 12 + m + 1;
			System.err.println("sssss=" + start.get(Calendar.DATE));//
			//			if((start.get(Calendar.DATE)==1) && (temp.get(Calendar.DATE)==1)){//前后都不破月 
			//				result= y*12+m+1; 
			//			} 
			//			else if((start.get(Calendar.DATE)!=1) && (temp.get(Calendar.DATE)==1)){//前破月后不破月 
			//				result= y*12+m; 
			//			} 
			//			else if((start.get(Calendar.DATE)==1) && (temp.get(Calendar.DATE)!=1)){//前不破月后破月 
			//				result= y*12+m; 
			//			} 
			//			else {//前破月后破月 	
			//		     result= (y*12+m-1)<0?0:(y*12+m-1); 
			//			}

			SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM");
			Date date = sdf.parse(data1);

			System.err.println("st0=" + sdf1.format(date));
			System.err.println("st=" + TimeHelper.getStrByaddMonth(data1, "yyyy-MM"));
			String[] res = new String[result];
			res[0] = sdf1.format(date);
			int i = 1;
			while (i < result) {
				res[i] = TimeHelper.getStrByaddMonth(res[i - 1], "yyyy-MM");
				i++;
			}
			for (int j = 0; j < res.length; j++) {//具体月
				System.err.println("str[" + j + "]=" + res[j]);//
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		System.err.println("result=" + result);
		return result;
	}

	/**
	 * @description 获取昨天的日期 日期格式 yyyy-MM-dd
	 * @return yyyy-MM-dd
	 * @return String
	 * 郭磊
	 */
	public static String getYesterdayDate() {
		GregorianCalendar calendar = new GregorianCalendar();
		calendar.add(GregorianCalendar.DATE, -1);
		String begin = new java.util.Date(calendar.getTime().getTime()).toString();
		return begin;
	}

	/**
	 * @description 获取前N周的起始日期 日期格式 yyyy-MM-dd
	 * @param lastNum 前N周 ,0为当前周,1为上周,2为上2周,依次类推
	 * @return yyyy-MM-dd
	 * 郭磊
	 */
	public static String getLastWeekStartDateByNum(int lastNum) {
		int num = 7 * ((lastNum < 0 ? 0 : lastNum) - 1);
		GregorianCalendar calendar = new GregorianCalendar();
		int minus = calendar.get(GregorianCalendar.DAY_OF_WEEK) + 1;
		calendar.add(GregorianCalendar.DATE, -minus - num);
		calendar.add(GregorianCalendar.DATE, -4);
		String begin = new java.sql.Date(calendar.getTime().getTime()).toString();
		return begin;
	}

	/**
	 * @description 获取本月的起始日期 日期格式 yyyy-MM-dd 
	 * @return yyyy-MM-dd
	 * 郭磊
	 */
	public static String getThisMonthStartDate() {
		GregorianCalendar calendar = new GregorianCalendar();
		int dayOfMonth = calendar.get(GregorianCalendar.DATE);
		calendar.add(GregorianCalendar.DATE, -dayOfMonth + 1);
		String begin = new java.sql.Date(calendar.getTime().getTime()).toString();
		return begin;
	}

	/**
	 * @description 获取上月的起始日期 日期格式 yyyy-MM-dd 
	 * @return yyyy-MM-dd
	 * 郭磊
	 */
	public static String getLastMonthStartDate() {
		GregorianCalendar calendar = new GregorianCalendar();
		calendar.set(calendar.get(GregorianCalendar.YEAR), calendar.get(GregorianCalendar.MONTH), 1);
		calendar.add(GregorianCalendar.DATE, -1);

		int month = calendar.get(GregorianCalendar.MONTH) + 1;
		String begin = calendar.get(GregorianCalendar.YEAR) + "-" + (month < 10 ? "0" + month : month) + "-01";
		return begin;
	}

	public static String getNextMonthStartDate() {
		GregorianCalendar calendar = new GregorianCalendar();
		calendar.set(calendar.get(GregorianCalendar.YEAR), calendar.get(GregorianCalendar.MONTH), 1);
		calendar.add(GregorianCalendar.DATE, +1);
		int month = calendar.get(GregorianCalendar.MONTH) + 1;
		System.out.println(GregorianCalendar.DATE);
		String begin = calendar.get(GregorianCalendar.YEAR) + "-" + (month < 10 ? "0" + month : month) + "-01";
		return begin;
	}

	/**
	 * @description 判断某日期是否在其它两个日期之间
	 * @param objDate 目标日期
	 * @param startDate 起始日期
	 * @param endDate 结束日期
	 * @return 是否在2个日期之间
	 * @return boolean
	 * @throws 比较失败
	 */
	public static boolean isBetweenDate(Date objDate, Date startDate, Date endDate) {
		return TimeHelper.isBetweenDate(objDate, startDate.getTime(), endDate.getTime());
	}

	/**
	 * @description 判断某日期是否在其它两个日期之间
	 * @param objDate 目标日期
	 * @param startDateTime 起始日期到今天毫秒数
	 * @param endDateTime 结束日期到今天毫秒数
	 * @return 是否在2个日期之间
	 * @return boolean
	 * @throws 比较失败
	 */
	public static boolean isBetweenDate(Date objDate, long startDateTime, long endDateTime) {
		long objDateTime = objDate.getTime();
		return objDateTime >= startDateTime && objDateTime <= endDateTime;
	}

	public static long getCurrentLongTime() {
		return new Date().getTime();

	}

	public static String getDateFormatForString(long time) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Date date = new Date(time);
		return sdf.format(date);
	}

	public static String getDateFormatForString(long time, String formate) {
		SimpleDateFormat sdf = new SimpleDateFormat(formate);
		Date date = new Date(time);
		return sdf.format(date);
	}

	/**
	 * 返回指定年、月、日、小时、分钟、秒所代表的时间值，以毫秒为单位
	 * 
	 * @param year 年
	 * @param month 月
	 * @param day 日
	 * @param hour 小时
	 * @param minute 分钟
	 * @param second 妙
	 * @return long 自1970年1月1日 00:00:00 GMT 以来此日期表示的毫秒数
	 * 苗康
	 */
	public static long getTime(int year, int month, int day, int hour, int minute, int second) {
		Calendar cd = Calendar.getInstance();
		cd.clear();
		cd.set(Calendar.YEAR, year);
		cd.set(Calendar.MONTH, month);
		cd.set(Calendar.DAY_OF_MONTH, day);
		cd.set(Calendar.HOUR_OF_DAY, hour);
		cd.set(Calendar.MINUTE, minute);
		cd.set(Calendar.SECOND, second);
		return cd.getTime().getTime();
	}

	/**
	 * 返回指定日期的开始时间，通常为 时间段为： 00:00:00
	 * 
	 * @param year 年
	 * @param month 月
	 * @param day 日
	 * @return 指定时间，以从历元至现在所经过的 UTC 毫秒数形式。
	 * 苗康
	 */
	public static long getDayBeginTime(int year, int month, int day) {
		return TimeHelper.getTime(year, month > 0 ? month - 1 : 0, day, 0, 0, 0);
	}

	/**
	 * 返回指定日期的最后时间，通常为 时间段为： 23:59:59
	 * 
	 * @param year 年
	 * @param month 月
	 * @param day 日
	 * @return 指定时间，以从历元至现在所经过的 UTC 毫秒数形式。
	 * 苗康
	 */
	public static long getDayEndTime(int year, int month, int day) {
		return TimeHelper.getTime(year, month > 0 ? month - 1 : 0, day, 23, 59, 59);
	}

	/**
	 * @description 获取某年某月的天数
	 * @param year 年
	 * @param month 月
	 * @return void
	 * @throws 获取失败
	 */
	public static Integer getMaxDayByYearMonth(Integer year, Integer month) {
		GregorianCalendar date = new GregorianCalendar(year, month - 1, 1);
		int number = date.getActualMaximum(GregorianCalendar.DATE);
		return number;
	}

	public static String getWeekDay(String currentdate) {
		GregorianCalendar gc = null;
		Calendar calendar = Calendar.getInstance();
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
		int wday;
		try {
			Date date = formatter.parse(currentdate);
			calendar.setTime(date);
			wday = calendar.get(Calendar.DAY_OF_WEEK) - 1;
			return new Integer(wday).toString();//addDay(currentdate,weekDay-wday);
		} catch (Exception e) {
			e.printStackTrace();
			return "";
		}
	}

	/**
	 * 方法说明 : 验证是否为时间字符串
	 * @param s
	 * @return
	 */
	public static boolean isDateTime(String s) {
		try {
			java.text.SimpleDateFormat sf = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			sf.setLenient(false);//这个的功能是不把1996-13-3 转换为1997-1-3
			sf.parse(s);
			return true;
		} catch (Exception ex) {
			return false;
		}
	}

	public static void main(String [] args){
	}
}
